// Class: EnvIterator
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.awt.Color;
import java.util.Iterator;

/**
 *  Environment-Based Applications:<br>
 *
 *    An EnvIterator object provides an iterator that that
 *    steps through the locations of a BoundedEnv object 
 *    in a particular order.
 *
 *    <p>
 *    EnvIterator implementations need not support the remove
 *    method, as it does not make sense to remove locations
 *    from a bounded environment.
 *
 *  @author Alyce Brady
 *  @version 1 September 2002
 *  @see BoundedEnv
 *  @see Location
 *  @see EnvDisplay
 **/
public abstract class EnvIterator implements Iterator
{
    // Encapsulated data for each EnvIterator object
    private BoundedEnv env;       // environment through which to iterate
    private Location nextLoc;     // next location in env

    /** Constructs an iterator object that steps through an environment
     *  starting at location (0, 0).
     *      @param environment     environment to iterate through
     **/
    public EnvIterator(BoundedEnv environment)
    {
        env = environment;
        nextLoc = new Location(0, 0);
    }

    /** Constructs an iterator object that steps through an environment,
     *  starting at a given location.
     *      @param environment     environment to iterate through
     *      @param startingLoc     where to start the iteration
     **/
    public EnvIterator(BoundedEnv environment, Location startingLoc)
    {
        env = environment;
        nextLoc = startingLoc;
    }

    /** Returns environment through which this iterator iterates.
     **/
    public BoundedEnv environment()
    {
        return env;
    }

    /** Returns true if the environment has more locations.
     **/
    public boolean hasNext()
    {
        // Return true if the next location is valid in the environment.
        return environment().isValid(nextLocation());
    }

    /** Returns next location (pure accessor -- does not advance iterator).
     **/
    protected Location nextLocation()
    {
        return nextLoc;
    }

    /** Returns the next location in the environment.
     **/
    public Object next()
    {
        Location locToReturn = nextLoc;
        nextLoc = findNextLocation();
        return locToReturn;
    }

    /** Removes from the underlying collection the last element
     *  returned by the iterator (not supported).
     **/
    public void remove()
    {
        throw new UnsupportedOperationException("remove method not supported");
    }

    /** Helper Method: Finds next location for iterator.
     **/
    protected abstract Location findNextLocation();

}
